{***************************************************************************}
{                             UNIDAD DE BOCINA

                                  Copyright (c) 1998 Marco Antonio Alvarado
}
{***************************************************************************}

UNIT UBocina;

INTERFACE

  TYPE
    PSonido = ^RSonido;

    RSonido = RECORD
      Frecuencia,
      Duracion : Word;
    END;

  VAR
    Silencio,
    Repite : Boolean;

  PROCEDURE Suena (Frecuencia : Word);
  PROCEDURE Calla;
  PROCEDURE PoneSonido (Sonido : Pointer; Cantidad : LongInt);
  PROCEDURE QuitaSonido;
  PROCEDURE TocaSonido;
  PROCEDURE ParaSonido;

IMPLEMENTATION

  USES
    UDS,  {Unidad de DOS Services}
    UMemoria;  {Unidad de memoria}

  CONST
{PIT = Programmable Interval Timer}
    FrecuenciaPIT = $1234dd;
    ControlPIT = $43;
    Canal2PIT = $42;
    PuertoBocina = $61;
    InterrupcionReloj = $1c;

  VAR
    ViejaAtiende,
    Inicio,
    Lugar : PSonido;
    Cantidad,
    Indice : LongInt;
    Frecuencia,
    Duracion : Word;
    Activo,
    Toca : Boolean;

  PROCEDURE Suena (Frecuencia : Word);

    VAR
      Cuenta : Word;

  BEGIN

    IF Frecuencia > 0 THEN
    BEGIN
{Programa el Programmable Interval Timer (PIT).}
{$r-}
      Cuenta := FrecuenciaPIT DIV Frecuencia;
{$r+}
      Port [ControlPIT] := $b6;
      Port [Canal2PIT] := Lo (Cuenta);
      Port [Canal2PIT] := Hi (Cuenta);

{Conecta la Bocina al PIT.}
      Port [PuertoBocina] := Port [PuertoBocina] OR 3;
    END ELSE
      IF Silencio THEN
      BEGIN
        Calla;
      END;

  END;

  PROCEDURE Calla;
  BEGIN
{Desconecta la Bocina del PIT.}
    Port [PuertoBocina] := Port [PuertoBocina] AND $fc;
  END;

{$F+}
  PROCEDURE Atiende; INTERRUPT;
{$F-}
  BEGIN

    IF Toca THEN
    BEGIN

      IF Duracion = 0 THEN
      BEGIN
        Frecuencia := Lugar^. Frecuencia;
        Duracion := Lugar^. Duracion;
        Suena (Frecuencia);
      END ELSE
      BEGIN
        Dec (Duracion);

        IF Duracion = 0 THEN
        BEGIN

          IF Indice < Cantidad - 1 THEN
          BEGIN
            Lugar := SumaPunteroCerca (Lugar, SizeOf (Lugar^));
            Inc (Indice);
          END ELSE
          BEGIN

            IF Repite THEN
            BEGIN
              Lugar := Inicio;
              Indice := 0;
            END ELSE
            BEGIN
              ParaSonido;
            END;

          END;

        END;

      END;

    END;

{Llama a la Vieja interrupcin.}
    ASM
      PUSHF
      CALL ViejaAtiende
    END;

  END;

  PROCEDURE PoneSonido (Sonido : Pointer; Cantidad : LongInt);
  BEGIN

    IF Toca THEN
    BEGIN
      IF Activo THEN QuitaSonido;
      Toca := True;
    END ELSE
    BEGIN
      IF Activo THEN QuitaSonido;
    END;

    Inicio := Sonido;
    Lugar := Inicio;
    UBocina. Cantidad := Cantidad;
    Indice := 0;
    Frecuencia := 0;
    Duracion := 0;

    ViejaAtiende := TomaInterrupcion (InterrupcionReloj);
    PoneInterrupcion (InterrupcionReloj, @Atiende);
    Activo := True;
  END;

  PROCEDURE QuitaSonido;
  BEGIN
    IF Toca THEN ParaSonido;

    IF Activo THEN
    BEGIN
      PoneInterrupcion (InterrupcionReloj, ViejaAtiende);
      Activo := False;
    END;

  END;

  PROCEDURE TocaSonido;
  BEGIN
    Toca := True;
  END;

  PROCEDURE ParaSonido;
  BEGIN
    Toca := False;
    Calla;
  END;

BEGIN
  Silencio := True;
  Repite := False;
  Activo := False;
  Toca := False;
END.